"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const orderBy_1 = tslib_1.__importDefault(require("lodash/orderBy"));
const mobx_1 = require("mobx");
const mobx_react_1 = require("mobx-react");
const http_1 = require("../helper/http");
const types_1 = require("../@typings/types");
const pagination_1 = require("../helper/pagination");
const field_1 = tslib_1.__importDefault(require("./models/field"));
const header_1 = tslib_1.__importDefault(require("./models/header"));
function useRootStore(props) {
    const observableProps = mobx_react_1.useAsObservableSource(props);
    return mobx_react_1.useLocalStore(() => ({
        props: observableProps,
        inProgress: false,
        _loadedData: undefined,
        currentPage: 0,
        selectedItems: {},
        sorting: {},
        error: undefined,
        focusCell: undefined,
        searchQuery: '',
        filterHandlers() {
            return [
                this.searchFilter,
                this.sortFilter
            ];
        },
        searchFilter(data) {
            if (this.searchQuery.length) {
                data = data.filter((row) => this.fields.some((field) => {
                    const searchable = this.headers && this.headers[field.index].searchable;
                    return searchable && row[field.property].toString().toLowerCase()
                        .indexOf(this.searchQuery.toLowerCase()) !== -1;
                }));
            }
            return data;
        },
        sortFilter(data) {
            if (Object.keys(this.sorting).length) {
                return orderBy_1.default(data, Object.keys(this.sorting), Object.values(this.sorting).map(v => v.order));
            }
            return data;
        },
        paginateFilter(data) {
            let _data = data;
            if (this.props.pagination && _data.length > this.props.pagination.pageSize && this.props.pagination.pageSize) {
                _data = pagination_1.paginate(data, this.props.pagination.pageSize, this.currentPage);
            }
            return { data: _data, pageCount: pagination_1.calculatePageCount(data, this.props.pagination) };
        },
        search(query) {
            this.searchQuery = query;
            this.currentPage = 0;
        },
        select(row) {
            if (this.props.selectMode) {
                const indexFieldValue = row[this.props.indexField];
                if (!this.selectedItems.hasOwnProperty(indexFieldValue)) {
                    if (this.props.selectMode === types_1.SelectModes.single) {
                        this.selectedItems = {};
                    }
                    this.selectedItems[indexFieldValue] = row;
                }
                else {
                    delete this.selectedItems[indexFieldValue];
                }
                if (this.props.onSelect) {
                    this.props.onSelect(Object.values(this.selectedItems));
                }
            }
        },
        get isLoading() {
            return this.inProgress && !this.error;
        },
        sort(header) {
            if (this.isLoading) {
                return;
            }
            const property = this.fields[header.index].property;
            const headerName = header.name;
            if (this.sorting[property]) {
                this.sorting[property].order = this.sorting[property].order === types_1.SortingValues.ASC ? types_1.SortingValues.DESC
                    : types_1.SortingValues.ASC;
            }
            else if (this.props.sorting === types_1.SortingModes.simple) {
                this.sorting = {
                    [property]: {
                        headerName,
                        order: types_1.SortingValues.ASC
                    }
                };
            }
            else {
                this.sorting[property] = {
                    headerName,
                    order: types_1.SortingValues.ASC
                };
            }
            this.currentPage = 0;
        },
        removeFromSorting(property) {
            delete this.sorting[property];
            this.currentPage = 0;
        },
        editCell(newValue, record, model) {
            if (record[model.property].toString() === newValue) {
                return;
            }
            record[model.property] = newValue;
            if (this.props.onEdit) {
                this.props.onEdit(newValue, model.property, mobx_1.toJS(record));
            }
        },
        isEditableField(field) {
            if (this.headers && this.headers.length) {
                const header = this.headers[field.index];
                if ((header.editable || this.props.editable) && field.renderHandler) {
                    console.warn(`Please avoid to use editable mode with custom render method for property: ${field.property}`);
                }
                if (typeof header.editable !== "undefined") {
                    return header.editable;
                }
            }
            return this.props.editable;
        },
        setFocus(cellIndex, rowIndex) {
            this.focusCell = { cellIndex, rowIndex };
        },
        moveFocus(keyCode) {
            if (this.focusCell) {
                switch (keyCode) {
                    case 37:
                        this.focusCell.cellIndex > 0 && this.focusCell.cellIndex--;
                        break;
                    case 38:
                        this.focusCell.rowIndex > 0 && this.focusCell.rowIndex--;
                        break;
                    case 39:
                        this.focusCell.cellIndex < this.fields.length - 1 && this.focusCell.cellIndex++;
                        break;
                    case 40:
                        this.focusCell.rowIndex < this.displayData.data.length - 1 && this.focusCell.rowIndex++;
                        break;
                    default:
                        return;
                }
            }
        },
        get displayData() {
            const filteredData = this.filterHandlers().reduce((f, g) => args => g(f(args)))(this._loadedData || this.props.data);
            return this.paginateFilter(filteredData);
        },
        get headers() {
            return this.props.headers && this.props.headers.map((header, index) => new header_1.default(header, index));
        },
        get fields() {
            return this.props.fields.map((field, index) => new field_1.default(field, index));
        },
        loadByUrl() {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                if (!this.props.url) {
                    return;
                }
                this.inProgress = true;
                try {
                    const res = yield http_1.load(this.props.url);
                    this.inProgress = false;
                    this._loadedData = (this.props.fetchSuccess ? this.props.fetchSuccess(res) : res);
                }
                catch (e) {
                    this.error = e.message;
                    console.error(e);
                }
                finally {
                    this.inProgress = false;
                }
            });
        },
    }));
}
exports.useRootStore = useRootStore;
//# sourceMappingURL=index.js.map